searchData={"items":[{"type":"module","title":"ContextKit","doc":"ContextKit provides a modular toolkit for building robust Phoenix/Ecto contexts with standardized CRUD operations.","ref":"ContextKit.html"},{"type":"module","title":"Overview - ContextKit","doc":"ContextKit aims to reduce boilerplate code in Phoenix applications by providing:\n\n- Standardized CRUD operation generators\n- Dynamic query building with extensive filtering options\n- Built-in pagination support\n- Flexible and extensible design","ref":"ContextKit.html#module-overview"},{"type":"module","title":"Getting Started - ContextKit","doc":"","ref":"ContextKit.html#module-getting-started"},{"type":"module","title":"1. First, define your schema module: - ContextKit","doc":"```elixir\ndefmodule MyApp.Accounts.User do\n  use Ecto.Schema\n\n  schema \"users\" do\n    field :email, :string\n    field :name, :string\n    field :status, :string\n\n    timestamps()\n  end\nend\n```","ref":"ContextKit.html#module-1-first-define-your-schema-module"},{"type":"module","title":"2. Create a queries module for custom query logic: - ContextKit","doc":"```elixir\ndefmodule MyApp.Accounts.UserQueries do\n  def apply_query_option({:with_active_posts, true}, query) do\n    query\n    |> join(:inner, [u], p in assoc(u, :posts))\n    |> where([_, p], p.status == \"active\")\n  end\nend\n```","ref":"ContextKit.html#module-2-create-a-queries-module-for-custom-query-logic"},{"type":"module","title":"3. Use ContextKit.CRUD in your context: - ContextKit","doc":"```elixir\ndefmodule MyApp.Accounts do\n  use ContextKit.CRUD,\n    repo: MyApp.Repo,\n    schema: MyApp.Accounts.User,\n    queries: MyApp.Accounts.UserQueries\nend\n```\n\nBy setting `queries: __MODULE__`, you can define your custom query functions (`apply_query_option/2`) directly in your context module,\n  eliminating the need for a separate queries module. This is particularly convenient for simpler contexts\n  where you don't need to share query logic across multiple modules.","ref":"ContextKit.html#module-3-use-contextkit-crud-in-your-context"},{"type":"module","title":"Features - ContextKit","doc":"","ref":"ContextKit.html#module-features"},{"type":"module","title":"Standard CRUD Operations - ContextKit","doc":"ContextKit automatically generates common CRUD functions:\n\n```elixir\n# List records with filtering and pagination\nAccounts.list_users(status: \"active\", paginate: [page: 1, per_page: 20])\n\n# Get single record\nAccounts.get_user(123)\nAccounts.get_user!(123)  # Raises if not found\n\n# Get one record by criteria\nAccounts.one_user(email: \"user@example.com\")\n\n# Delete records\nAccounts.delete_user(user)\nAccounts.delete_user(email: \"user@example.com\")\n```","ref":"ContextKit.html#module-standard-crud-operations"},{"type":"module","title":"Advanced Filtering - ContextKit","doc":"Supports a wide range of filter operations:\n\n```elixir\n# Basic equality\nAccounts.list_users(status: \"active\")\n\n# Complex filters\nAccounts.list_users(filters: [\n  %{field: :email, op: :ilike, value: \"@gmail.com\"},\n  %{field: :status, op: :in, value: [\"active\", \"pending\"]},\n  %{field: :name, op: :like_or, value: [\"john\", \"jane\"]}\n])\n```\n\nAny option not recognized as a field filter or standard query option is treated as a custom\n  query option and passed to the queries module's `apply_query_option/2` function.","ref":"ContextKit.html#module-advanced-filtering"},{"type":"module","title":"Pagination - ContextKit","doc":"Built-in pagination support:\n\n```elixir\n{users, pagination} = Accounts.list_users(\n  status: \"active\",\n  paginate: [page: 2, per_page: 20]\n)\n\n# pagination struct includes:\n# - total_count\n# - total_pages\n# - current_page\n# - per_page\n# - has_next_page?\n# - has_previous_page?\n# - next_page\n# - previous_page\n```","ref":"ContextKit.html#module-pagination"},{"type":"module","title":"Custom Query Options - ContextKit","doc":"Any option not recognized as a field filter or standard query option is treated as a custom\n  query option and passed to the queries module's `apply_query_option/2` function.\n\nExtend with custom query logic:\n\n```elixir\n# In your queries module\ndef apply_query_option({:with_recent_activity, true}, query) do\n  query\n  |> where([u], u.last_active_at > ago(1, \"day\"))\nend\n\n# Usage\nAccounts.list_users(with_recent_activity: true)\n```","ref":"ContextKit.html#module-custom-query-options"},{"type":"module","title":"Configuration Options - ContextKit","doc":"When using `ContextKit.CRUD`, you can configure:\n\n- `repo`: Your Ecto repository module\n- `schema`: The Ecto schema module\n- `queries`: Module containing custom query functions\n- `except`: List of operations to exclude (`:list`, `:get`, `:one`, `:delete`)\n- `plural_resource_name`: Custom plural name for list functions","ref":"ContextKit.html#module-configuration-options"},{"type":"module","title":"Best Practices - ContextKit","doc":"1. Create separate query modules for complex filtering logic\n2. Override generated functions when you need custom behavior\n3. Use pagination for large datasets\n4. Leverage custom query options for reusable query logic","ref":"ContextKit.html#module-best-practices"},{"type":"module","title":"ContextKit.CRUD","doc":"The `ContextKit.CRUD` module provides a convenient way to generate standard CRUD (Create, Read, Update, Delete)\noperations for your Ecto schemas. It reduces boilerplate code by automatically generating commonly used database\ninteraction functions.","ref":"ContextKit.CRUD.html"},{"type":"module","title":"Setup - ContextKit.CRUD","doc":"Add the following to your context module:\n\n```elixir\ndefmodule MyApp.Accounts do\n  use ContextKit.CRUD,\n    repo: MyApp.Repo,\n    schema: MyApp.Accounts.User,\n    queries: MyApp.Accounts.UserQueries,\n    except: [:delete],                    # Optional: exclude specific operations\n    plural_resource_name: \"users\"         # Optional: customize plural name\nend\n```","ref":"ContextKit.CRUD.html#module-setup"},{"type":"module","title":"Required Options - ContextKit.CRUD","doc":"* `:repo` - The Ecto repository module to use for database operations\n  * `:schema` - The Ecto schema module that defines your resource\n  * `:queries` - Module containing query-building functions for advanced filtering","ref":"ContextKit.CRUD.html#module-required-options"},{"type":"module","title":"Optional Options - ContextKit.CRUD","doc":"* `:except` - List of operation types to exclude (`:list`, `:get`, `:one`, `:delete`, `:create`, `:update`, `:change`)\n  * `:plural_resource_name` - Custom plural name for list functions (defaults to singular + \"s\")","ref":"ContextKit.CRUD.html#module-optional-options"},{"type":"module","title":"Generated Functions - ContextKit.CRUD","doc":"For a schema named `User`, the following functions are generated:","ref":"ContextKit.CRUD.html#module-generated-functions"},{"type":"module","title":"List Operations - ContextKit.CRUD","doc":"* `list_users/0` - Returns all users\n  * `list_users/1` - Returns filtered users based on options","ref":"ContextKit.CRUD.html#module-list-operations"},{"type":"module","title":"Get Operations - ContextKit.CRUD","doc":"* `get_user/1` - Fetches a single user by ID\n  * `get_user/2` - Fetches a user by ID with additional filters\n  * `get_user!/1` - Like `get_user/1` but raises if not found\n  * `get_user!/2` - Like `get_user/2` but raises if not found","ref":"ContextKit.CRUD.html#module-get-operations"},{"type":"module","title":"Single Record Operations - ContextKit.CRUD","doc":"* `one_user/1` - Fetches a single user matching the criteria\n  * `one_user!/1` - Like `one_user/1` but raises if not found","ref":"ContextKit.CRUD.html#module-single-record-operations"},{"type":"module","title":"Create Operations - ContextKit.CRUD","doc":"* `create_user/1` - Creates a new user with provided attributes\n  * `create_user!/1` - Like `create_user/1` but raises on invalid attributes","ref":"ContextKit.CRUD.html#module-create-operations"},{"type":"module","title":"Update Operations - ContextKit.CRUD","doc":"* `update_user/2` - Updates user with provided attributes\n  * `update_user!/2` - Like `update_user/2` but raises on invalid attributes","ref":"ContextKit.CRUD.html#module-update-operations"},{"type":"module","title":"Change Operations - ContextKit.CRUD","doc":"* `change_user/2` - Returns a changeset for the user with optional changes","ref":"ContextKit.CRUD.html#module-change-operations"},{"type":"module","title":"Delete Operations - ContextKit.CRUD","doc":"* `delete_user/1` - Deletes a user struct or by query criteria","ref":"ContextKit.CRUD.html#module-delete-operations"},{"type":"module","title":"Query Options - ContextKit.CRUD","doc":"All functions that accept options support:\n\n  * Basic filtering with field-value pairs\n  * Complex queries via `Ecto.Query`\n  * Pagination via `paginate: true` or `paginate: [page: 1, per_page: 20]`\n  * Custom query options defined in your queries module","ref":"ContextKit.CRUD.html#module-query-options"},{"type":"module","title":"Examples - ContextKit.CRUD","doc":"```elixir\n# List all users\nMyApp.Accounts.list_users()\n\n# List active users with pagination\nMyApp.Accounts.list_users(status: :active, paginate: [page: 1])\n\n# Get user by ID with preloads\nMyApp.Accounts.get_user(123, preload: [:posts])\n\n# Create a new user\nMyApp.Accounts.create_user(%{email: \"new@example.com\"})\n\n# Update a user\nMyApp.Accounts.update_user(user, %{email: \"updated@example.com\"})\n\n# Get a changeset for updates\nMyApp.Accounts.change_user(user, %{email: \"changed@example.com\"})\n\n# Delete user matching criteria\nMyApp.Accounts.delete_user(email: \"user@example.com\")\n```\n\nEach generated function can be overridden in your context module if you need custom behavior.","ref":"ContextKit.CRUD.html#module-examples"},{"type":"module","title":"ContextKit.Paginator","doc":"Handles pagination for database queries in ContextKit.\n\nThis module provides pagination functionality with configurable page size,\nautomatic limit/offset calculation, and metadata about the paginated results.","ref":"ContextKit.Paginator.html"},{"type":"function","title":"ContextKit.Paginator.changed?/2","doc":"","ref":"ContextKit.Paginator.html#changed?/2"},{"type":"function","title":"ContextKit.Paginator.new/3","doc":"","ref":"ContextKit.Paginator.html#new/3"},{"type":"function","title":"ContextKit.Paginator.params/1","doc":"","ref":"ContextKit.Paginator.html#params/1"},{"type":"module","title":"ContextKit.Query","doc":"Provides dynamic query building functionality with extensive filtering operations for Ecto queries.\n\nThis module is core to ContextKit's filtering capabilities, providing a flexible way to build\ncomplex database queries from runtime parameters.","ref":"ContextKit.Query.html"},{"type":"module","title":"Query Building - ContextKit.Query","doc":"The query building process consists of:\n1. Creating a base query with proper binding names\n2. Applying field filters based on schema fields\n3. Applying standard query options (order_by, limit, etc.)\n4. Applying custom query options","ref":"ContextKit.Query.html#module-query-building"},{"type":"module","title":"Field Filters - ContextKit.Query","doc":"Supports an extensive set of filter operations through a standardized filter syntax:\n\n```elixir\n%{field: field_name, op: operator, value: value}\n```","ref":"ContextKit.Query.html#module-field-filters"},{"type":"module","title":"Available Operators - ContextKit.Query","doc":"Basic Comparison:\n- `:==` - Equality\n- `:!=` - Inequality\n- `:<` - Less than\n- `:<=` - Less than or equal\n- `:>` - Greater than\n- `:>=` - Greater than or equal\n\nNull Checks:\n- `:empty` - Check if field is NULL\n- `:not_empty` - Check if field is NOT NULL\n\nList Operations:\n- `:in` - Check if field value is in list\n- `:not_in` - Check if field value is not in list\n- `:contains` - Check if list field contains value\n- `:not_contains` - Check if list field does not contain value\n\nPattern Matching:\n- `:like` - Case-sensitive pattern matching\n- `:not_like` - Negative case-sensitive pattern matching\n- `:ilike` - Case-insensitive pattern matching\n- `:not_ilike` - Negative case-insensitive pattern matching\n- `:=~` - Shorthand for `:ilike`\n\nMultiple Pattern Matching:\n- `:like_and` - All patterns must match (case-sensitive)\n- `:like_or` - Any pattern must match (case-sensitive)\n- `:ilike_and` - All patterns must match (case-insensitive)\n- `:ilike_or` - Any pattern must match (case-insensitive)","ref":"ContextKit.Query.html#module-available-operators"},{"type":"module","title":"Standard Query Options - ContextKit.Query","doc":"The following standard query options are supported:\n\n  - `:order_by` - Specify sort order\n  - `:group_by` - Group results\n  - `:limit` - Limit number of results\n  - `:preload` - Preload associations\n  - `:paginate` - Enable pagination with optional configuration","ref":"ContextKit.Query.html#module-standard-query-options"},{"type":"function","title":"ContextKit.Query.build/3","doc":"","ref":"ContextKit.Query.html#build/3"},{"type":"function","title":"ContextKit.Query.field_filter_keys/1","doc":"","ref":"ContextKit.Query.html#field_filter_keys/1"},{"type":"function","title":"ContextKit.Query.new/1","doc":"","ref":"ContextKit.Query.html#new/1"},{"type":"function","title":"ContextKit.Query.query_options_keys/1","doc":"","ref":"ContextKit.Query.html#query_options_keys/1"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}